/**
 * \file        lzma/info.h
 * \brief       Handling of Stream size information
 *
 * \author      Copyright (C) 1999-2006 Igor Pavlov
 * \author      Copyright (C) 2007 Lasse Collin
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#ifndef LZMAALPHA_H_INTERNAL
#	error Never include this file directly. Use <lzma.h> instead.
#endif


/**********
 * Basics *
 **********/

/**
 * \brief       Opaque data type to hold the size information
 */
typedef struct lzmaalpha_info_s lzmaalpha_info;


typedef struct {
	/**
	 * \brief       Total Size of this Block
	 *
	 * This can be LZMAALPHA_VLI_VALUE_UNKNOWN.
	 */
	lzmaalpha_vli total_size;

	/**
	 * \brief       Uncompressed Size of this Block
	 *
	 * This can be LZMAALPHA_VLI_VALUE_UNKNOWN.
	 */
	lzmaalpha_vli uncompressed_size;

	/**
	 * \brief       Offset of the first byte of the Block
	 *
	 * In encoder, this is useful to find out the alignment of the Block.
	 *
	 * In decoder, this is useful when doing random-access reading
	 * with help from lzmaalpha_info_data_locate().
	 */
	lzmaalpha_vli stream_offset;

	/**
	 * \brief       Uncompressed offset of the Block
	 *
	 * Offset of the first uncompressed byte of the Block relative to
	 * all uncompressed data in the Block.
	 * FIXME desc
	 */
	lzmaalpha_vli uncompressed_offset;

	/**
	 * \brief       Pointers to internal data structures
	 *
	 * Applications must not touch these.
	 */
	void *internal[4];

} lzmaalpha_info_iter;


typedef enum {
	LZMAALPHA_INFO_STREAM_START,
	LZMAALPHA_INFO_HEADER_METADATA,
	LZMAALPHA_INFO_TOTAL,
	LZMAALPHA_INFO_UNCOMPRESSED,
	LZMAALPHA_INFO_FOOTER_METADATA
} lzmaalpha_info_size;


/**
 * \brief       Allocates and initializes a new lzmaalpha_info structure
 *
 * If info is NULL, a new lzmaalpha_info structure is allocated, initialized, and
 * a pointer to it returned. If allocation fails, NULL is returned.
 *
 * If info is non-NULL, it is reinitialized and the same pointer returned.
 * (In this case, return value cannot be NULL or a different pointer than
 * the info given as argument.)
 */
extern lzmaalpha_info *lzmaalpha_info_init(lzmaalpha_info *info, lzmaalpha_allocator *allocator);


/**
 * \brief       Resets lzmaalpha_info
 *
 * This is like calling lzmaalpha_info_end() and lzmaalpha_info_create(), but
 * re-uses the existing base structure.
 */
extern void lzmaalpha_info_reset(
		lzmaalpha_info *info, lzmaalpha_allocator *allocator);


/**
 * \brief       Frees memory allocated for a lzmaalpha_info structure
 */
extern void lzmaalpha_info_free(lzmaalpha_info *info, lzmaalpha_allocator *allocator);


/************************
 * Setting known values *
 ************************/

/**
 * \brief       Set a known size value
 *
 * \param       info    Pointer returned by lzmaalpha_info_create()
 * \param       type    Any value from lzmaalpha_info_size
 * \param       size    Value to set or verify
 *
 * \return      LZMAALPHA_OK on success, LZMAALPHA_DATA_ERROR if the size doesn't
 *              match the existing information, or LZMAALPHA_PROG_ERROR
 *              if type is invalid or size is not a valid VLI.
 */
extern lzmaalpha_ret lzmaalpha_info_size_set(
		lzmaalpha_info *info, lzmaalpha_info_size type, lzmaalpha_vli size);


/**
 * \brief       Sets the Index
 *
 * The given lzmaalpha_index list is "absorbed" by this function. The application
 * must not access it after this function call, even if this function returns
 * an error.
 *
 * \note        The given lzmaalpha_index will at some point get freed by the
 *              lzma_info_* functions. If you use a custom lzmaalpha_allocator,
 *              make sure that it can free the lzmaalpha_index.
 */
extern lzmaalpha_ret lzmaalpha_info_index_set(
		lzmaalpha_info *info, lzmaalpha_allocator *allocator,
		lzmaalpha_index *index, lzmaalpha_bool eat_index);


/**
 * \brief       Sets information from a known Metadata Block
 *
 * This is a shortcut for calling lzmaalpha_info_size_set() with different type
 * arguments, lzmaalpha_info_index_set() with metadata->index.
 */
extern lzmaalpha_ret lzmaalpha_info_metadata_set(lzmaalpha_info *info,
		lzmaalpha_allocator *allocator, lzmaalpha_metadata *metadata,
		lzmaalpha_bool is_header_metadata, lzmaalpha_bool eat_index);


/***************
 * Incremental *
 ***************/

/**
 * \brief       Prepares an iterator to be used with given lzmaalpha_info structure
 *
 *
 */
extern void lzmaalpha_info_iter_begin(lzmaalpha_info *info, lzmaalpha_info_iter *iter);


/**
 * \brief       Moves to the next Index Record
 *
 *
 */
extern lzmaalpha_ret lzmaalpha_info_iter_next(
		lzmaalpha_info_iter *iter, lzmaalpha_allocator *allocator);


/**
 * \brief       Sets or verifies the sizes in the Index Record
 *
 * \param       iter    Pointer to iterator to be set or verified
 * \param       total_size
 *                      Total Size in bytes or LZMAALPHA_VLI_VALUE_UNKNOWN
 * \param       uncompressed_size
 *                      Uncompressed Size or LZMAALPHA_VLI_VALUE_UNKNOWN
 *
 * \return      - LZMAALPHA_OK: All OK.
 *              - LZMAALPHA_DATA_ERROR: Given sizes don't match with the already
 *                known sizes.
 *              - LZMAALPHA_PROG_ERROR: Internal error, possibly integer
 *                overflow (e.g. the sum of all the known sizes is too big)
 */
extern lzmaalpha_ret lzmaalpha_info_iter_set(lzmaalpha_info_iter *iter,
		lzmaalpha_vli total_size, lzmaalpha_vli uncompressed_size);


/**
 * \brief       Locates a Data Block
 *
 * \param       iter        Properly initialized iterator
 * \param       allocator   Pointer to lzmaalpha_allocator or NULL
 * \param       uncompressed_offset
 *                          Target offset to locate. The final offset
 *                          will be equal or smaller than this.
 * \param       allow_alloc True if this function is allowed to call
 *                          lzmaalpha_info_iter_next() to allocate a new Record
 *                          if the requested offset reached end of Index
 *                          Record list. Note that if Index has been marked
 *                          final, lzmaalpha_info_iter_next() is never called.
 *
 * \return      - LZMAALPHA_OK: All OK, *iter updated accordingly.
 *              - LZMAALPHA_DATA_ERROR: Trying to search past the end of the Index
 *                Record list, and allocating a new Record was not allowed
 *                either because allow_alloc was false or Index was final.
 *              - LZMAALPHA_PROG_ERROR: Internal error (probably integer
 *                overflow causing some lzmaalpha_vli getting too big).
 */
extern lzmaalpha_ret lzmaalpha_info_iter_locate(lzmaalpha_info_iter *iter,
		lzmaalpha_allocator *allocator, lzmaalpha_vli uncompressed_offset,
		lzmaalpha_bool allow_alloc);


/**
 * \brief       Finishes incrementally constructed Index
 *
 * This sets the known Total Size and Uncompressed of the Data Blocks
 * based on the information collected from the Index Records, and marks
 * the Index as final.
 */
extern lzmaalpha_ret lzmaalpha_info_index_finish(lzmaalpha_info *info);


/***************************
 * Reading the information *
 ***************************/

/**
 * \brief       Gets a known size
 *
 *
 */
extern lzmaalpha_vli lzmaalpha_info_size_get(
		const lzmaalpha_info *info, lzmaalpha_info_size type);

extern lzmaalpha_vli lzmaalpha_info_metadata_locate(
		const lzmaalpha_info *info, lzmaalpha_bool is_header_metadata);

/**
 * \brief       Gets a pointer to the beginning of the Index list
 *
 * If detach is true, the Index will be detached from the lzmaalpha_info
 * structure, and thus not be modified or freed by lzmaalpha_info_end().
 *
 * If detach is false, the application must not modify the Index in any way.
 * Also, the Index list is guaranteed to be valid only till the next call
 * to any lzma_info_* function.
 */
extern lzmaalpha_index *lzmaalpha_info_index_get(lzmaalpha_info *info, lzmaalpha_bool detach);


extern size_t lzmaalpha_info_index_count_get(const lzmaalpha_info *info);


extern uint32_t lzmaalpha_info_metadata_alignment_get(
		const lzmaalpha_info *info, lzmaalpha_bool is_header_metadata);



/**
 * \brief       Locate a Block containing the given uncompressed offset
 *
 * This function is useful when you need to do random-access reading in
 * a Multi-Block Stream.
 *
 * \param       info    Pointer to lzmaalpha_info that has at least one
 *                      Index Record. The Index doesn't need to be finished.
 * \param       uncompressed_target
 *                      Uncompressed target offset which the caller would
 *                      like to locate from the Stream.
 * \param       stream_offset
 *                      Starting offset (relative to the beginning the Stream)
 *                      of the Block containing the requested location.
 * \param       uncompressed_offset
 *                      The actual uncompressed offset of the beginning of
 *                      the Block. uncompressed_offset <= uncompressed_target
 *                      is always true; the application needs to uncompress
 *                      uncompressed_target - uncompressed_offset bytes to
 *                      reach the requested target offset.
 * \param       total_size
 *                      Total Size of the Block. If the Index is incomplete,
 *                      this may be LZMAALPHA_VLI_VALUE_UNKNOWN indicating unknown
 *                      size.
 * \param       uncompressed_size
 *                      Uncompressed Size of the Block. If the Index is
 *                      incomplete, this may be LZMAALPHA_VLI_VALUE_UNKNOWN
 *                      indicating unknown size. The application must pass
 *                      this value to the Block decoder to verify FIXME
 *
 * \return
 *
 * \note        This function is currently implemented as a linear search.
 *              If there are many Index Records, this can be really slow.
 *              This can be improved in newer liblzma versions if needed.
 */
extern lzmaalpha_bool lzmaalpha_info_data_locate(const lzmaalpha_info *info,
		lzmaalpha_vli uncompressed_target,
		lzmaalpha_vli *lzmaalpha_restrict stream_offset,
		lzmaalpha_vli *lzmaalpha_restrict uncompressed_offset,
		lzmaalpha_vli *lzmaalpha_restrict total_size,
		lzmaalpha_vli *lzmaalpha_restrict uncompressed_size);
