///////////////////////////////////////////////////////////////////////////////
//
/// \file       tests.h
/// \brief      Common definitions for test applications
//
//  Copyright (C) 2006 Lasse Collin
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef LZMA_TESTS_H
#define LZMA_TESTS_H

#include "sysdefs.h"

#include <stdio.h>

#define memcrap(buf, size) memset(buf, 0xFD, size)

#define expect(test) ((test) ? 0 : (fprintf(stderr, "%s:%u: %s\n", \
	__FILE__, __LINE__, #test), exit(1), 0))

#define succeed(test) expect(!(test))

#define fail(test) expect(test)


static inline const char *
lzma_ret_sym(lzmaalpha_ret ret)
{
	const char *str = "";

	switch (ret) {
	case LZMAALPHA_OK:
		str = "LZMAALPHA_OK";
		break;

	case LZMAALPHA_STREAM_END:
		str = "LZMAALPHA_STREAM_END";
		break;

	case LZMAALPHA_PROG_ERROR:
		str = "LZMAALPHA_PROG_ERROR";
		break;

	case LZMAALPHA_DATA_ERROR:
		str = "LZMAALPHA_DATA_ERROR";
		break;

	case LZMAALPHA_MEM_ERROR:
		str = "LZMAALPHA_MEM_ERROR";
		break;

	case LZMAALPHA_BUF_ERROR:
		str = "LZMAALPHA_BUF_ERROR";
		break;

	case LZMAALPHA_HEADER_ERROR:
		str = "LZMAALPHA_HEADER_ERROR";
		break;

	case LZMAALPHA_UNSUPPORTED_CHECK:
		str = "LZMAALPHA_UNSUPPORTED_CHECK";
		break;
	}

	return str;
}


static inline bool
coder_loop(lzmaalpha_stream *strm, uint8_t *in, size_t in_size,
		uint8_t *out, size_t out_size,
		lzmaalpha_ret expected_ret, lzmaalpha_action finishing_action)
{
	size_t in_left = in_size;
	size_t out_left = out_size > 0 ? out_size + 1 : 0;
	lzmaalpha_action action = LZMAALPHA_RUN;
	lzmaalpha_ret ret;

	strm->next_in = NULL;
	strm->avail_in = 0;
	strm->next_out = NULL;
	strm->avail_out = 0;

	while (true) {
		if (in_left > 0) {
			if (--in_left == 0)
				action = finishing_action;

			strm->next_in = in++;
			strm->avail_in = 1;
		}

		if (out_left > 0) {
			--out_left;
			strm->next_out = out++;
			strm->avail_out = 1;
		}

		ret = lzmaalpha_code(strm, action);
		if (ret != LZMAALPHA_OK)
			break;
	}

	bool error = false;

	if (ret != expected_ret)
		error = true;

	if (expected_ret == LZMAALPHA_STREAM_END) {
		if (strm->total_in != in_size || strm->total_out != out_size)
			error = true;
	} else {
		if (strm->total_in + 1 != in_size
				|| strm->total_out != out_size)
			error = true;
	}

	return error;
}


static inline bool
decoder_loop_ret(lzmaalpha_stream *strm, uint8_t *in, size_t in_size,
		lzmaalpha_ret expected_ret)
{
	return coder_loop(strm, in, in_size, NULL, 0, expected_ret, LZMAALPHA_RUN);
}


static inline bool
decoder_loop(lzmaalpha_stream *strm, uint8_t *in, size_t in_size)
{
	return coder_loop(strm, in, in_size, NULL, 0,
			LZMAALPHA_STREAM_END, LZMAALPHA_RUN);
}

#endif
